@@ -19,12 +19,12 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /usr/src/distr/ftape-0.9.10/RCS/calibr.h,v $
+ $Source: /usr/src/distr/ftape-1.12/RCS/calibr.h,v $
  $Author: bas $
  *
- $Revision: 1.11 $
- $Date: 1994/02/20 15:26:55 $
- $State: Alpha $
+ $Revision: 1.13 $
+ $Date: 1994/05/26 23:43:45 $
+ $State: Beta $
  *
  *      This file contains a gp calibration routine for
  *      hardware dependent timeout functions.
