@@ -15,18 +15,18 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-0.9.10/RCS/fdc-io.c,v $
+ $Source: /usr/src/distr/ftape-1.12/RCS/fdc-io.c,v $
  $Author: bas $
  *
- $Revision: 1.24 $
- $Date: 1994/02/20 15:26:55 $
- $State: Alpha $
+ $Revision: 1.26 $
+ $Date: 1994/05/26 23:43:45 $
+ $State: Beta $
  *
  *      This file contains the low-level floppy disk interface code
  *      for the QIC-40/80 tape streamer device driver.
  */
 
-static char RCSid[] = "$Id: fdc-io.c,v 1.24 1994/02/20 15:26:55 bas Alpha bas $";
+static char RCSid[] = "$Id: fdc-io.c,v 1.26 1994/05/26 23:43:45 bas Beta bas $";
 
 #include <linux/errno.h>
 #include <linux/sched.h>
@@ -298,22 +298,23 @@ fdc_issue_command( byte *out_data, int out_count,
 }
 
 /*      Wait for FDC interrupt with timeout.
+ *      Note: interrupts must be enabled ! (23/05/93 SJL)
  */
 int
 fdc_interrupt_wait( int time) {
   TRACE_FUN( 8, "fdc_interrupt_wait");
-  unsigned long flags;
   struct wait_queue wait = { current, NULL };
   int result = -ETIME;
+  int need_cleanup = 0;
+  int current_blocked = current->blocked;
   
   if (wait_intr) {
     TRACE( 1, "error: nested call");
     return -EIO;                /* return error... */
   }
-  save_flags( flags);
-  cli();
   if (interrupt_seen == 0) {
     int signal_pending = 0;
+
    /*   timeout time will be between 0 and MSPT milliseconds too long ! */
     current->timeout = jiffies + 1 + (time + MSPT - 1) / MSPT;
     current->state = TASK_INTERRUPTIBLE;
@@ -321,48 +322,43 @@ fdc_interrupt_wait( int time) {
     add_wait_queue( &wait_intr, &wait);
   
     while (1) {
-      sti();                    /* allow interrupt_seen */
       while (!interrupt_seen && current->state != TASK_RUNNING) {
         schedule();             /* sets TASK_RUNNING on timeout */
       }
-      cli();                    /* prevents late interrupt_seen */
       if (current->signal & ~current->blocked) {
-        /* probably awakened by non-blocked signal, fdc probably still busy.
-         * will create a mess if we don't reset it !
-         */
-#if 0
-        disable_dma( FLOPPY_DMA);
-        fdc_reset();
-#endif
-        /* hack (?) to allow gracefull exit
-         * just mask every signal that interferes
+        /* awoken by signal, block it to be able to finish the wait
+         * but remember that we got a signal and must abort.
          */
         current->blocked |= (current->signal & ~current->blocked) ;
         signal_pending = 1;
       } else {
-        if (signal_pending) {
-          TRACE( 1, "error: aborted on signal");
-          current->timeout = 0;
-          remove_wait_queue( &wait_intr, &wait);
-          restore_flags( flags);
-          TRACE_EXIT;
-          return -EINTR;
-        } else {
-          break;
-        }
+        break;
       }
     }
-    /* get here only on timeout or qic117 interrupt, TASK_RUNNING */
     if (interrupt_seen) {
+      /* got interrupt: good */
       current->timeout = 0;     /* interrupt hasn't cleared this */
       result = 0;
+    } else {
+      if (signal_pending) {
+        /* didn't get interrupt but got signal (& timeout): abort */
+        current->timeout = 0;   /* interrupt hasn't cleared this */
+        result = -EINTR;
+      } else {
+        /* didn't get interrupt nor signal: timeout */
+        result = -ETIME;
+      }
+      need_cleanup = 1;
     }
     remove_wait_queue( &wait_intr, &wait);
   } else {
     result = 0;
   }
-  restore_flags( flags);
-  interrupt_seen = 0;		/* should be somewhere else ! */
+  current->blocked = current_blocked; /* restore old state */
+  interrupt_seen = 0;		/* clear for next call */
+  if (need_cleanup) {
+    fdc_reset();
+  }
   TRACE_EXIT;
   return result;
 }
@@ -396,24 +392,29 @@ fdc_reset(void)
   TRACE_FUN( 8, "fdc_reset");
   int unit = FTAPE_UNIT;
   byte fdc_ctl = unit | FDC_DMA_MODE;
+  int st0;
+  int i;
+  int result;
 
-  expect_stray_interrupt = 1;
   if (ftape_motor) {
     fdc_ctl |= FDC_MOTOR_0 << unit;
   }
-  /* Assert the reset line.  Leave the proper unit selected.
-   */
-  outb_p( fdc_ctl, FDC_CONTROL);
-  ftape_sleep( MILLISECOND);    /* > 14 fdc clocks */
-  /* Now release the reset line.
-   */
+  outb_p( fdc_ctl, FDC_CONTROL); /* assert reset, keep unit selected */
+  fdc_usec_wait( 10 /* usec */); /* delay >= 14 fdc clocks */
   fdc_ctl |= FDC_RESET_NOT;
-  outb_p( fdc_ctl, FDC_CONTROL);
-  ftape_sleep( 10 * MILLISECOND); /* ??? */
+  outb_p( fdc_ctl, FDC_CONTROL); /* release reset */
+  result = fdc_interrupt_wait( 1 * SECOND);
+  if (result < 0) {
+    TRACE( 1, "missing interrupt after reset");
+  }
   /* Select clock for fdc.  May need to be changed to select data rate.
    */
   outb_p( 0, FDC_VFO);          /* set high density (500 Kbps) mode */
-  expect_stray_interrupt = 0;
+  fdc_usec_wait( 1000 /* usec */); /* don't know why, but needed */
+  for (i = 0 ; i < 4 ; ++i) {   /* clear disk-change status */
+    fdc_sense_interrupt_status( &st0, &current_cylinder);
+  }
+  fdc_set_seek_rate( 2);
   TRACE_EXIT;
 }
 
@@ -511,7 +512,6 @@ fdc_seek( int track)
     /*  We really need this command to work !
      */
     fdc_reset();
-    fdc_set_seek_rate( 2);
     TRACE_EXIT;
     return result;
   }
