@@ -15,18 +15,18 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-0.9.10/RCS/fdc-isr.c,v $
+ $Source: /usr/src/distr/ftape-1.12/RCS/fdc-isr.c,v $
  $Author: bas $
  *
- $Revision: 1.7 $
- $Date: 1994/02/20 15:26:55 $
- $State: Alpha $
+ $Revision: 1.9 $
+ $Date: 1994/05/26 23:43:45 $
+ $State: Beta $
  *
  *      This file contains the interrupt service routine and associated
  *      code for the QIC-40/80 tape streamer device driver.
  */
 
-static char RCSid[] = "$Id: fdc-isr.c,v 1.7 1994/02/20 15:26:55 bas Alpha bas $";
+static char RCSid[] = "$Id: fdc-isr.c,v 1.9 1994/05/26 23:43:45 bas Beta bas $";
 
 #include <asm/io.h>
 #include <asm/dma.h>
@@ -500,7 +500,11 @@ fdc_isr( void)
           case overrun_error: {
             update_history( cause);
             if (buff->crc_error_map == 0) {
-              TRACE( 4, "single data crc or overrun error");
+              if (cause == data_crc_error) {
+                TRACE( 4, "single crc error in data");
+              } else {
+                TRACE( 4, "single overrun error");
+              }
               continue_xfer( &buff, cause, 0, 0);
             } else {
               TRACE( 4, "too many errors, must retry");
@@ -511,7 +515,7 @@ fdc_isr( void)
           default: {
             /*  Don't know why this could happen but find out.
              */
-            TRACE( 1, "unexpected error, see state above");
+            TRACE( 1, "unexpected error");
             retry_sector( &buff, cause, 0, 0);
             break;
           }
@@ -582,6 +586,13 @@ fdc_isr( void)
           case id_crc_error:
           case data_am_error: {
             update_history( cause);
+            TRACE( 4, "single am/crc error");
+            retry_sector( &buff, cause, 1, buff->sector_offset);
+            break;
+          }
+          case overrun_error: {
+            update_history( cause);
+            TRACE( 4, "single overrun error");
             retry_sector( &buff, cause, 1, buff->sector_offset);
             break;
           }
@@ -634,6 +645,8 @@ fdc_isr( void)
         TRACE( 5, "expected stray interrupt");
         --expected_stray_interrupts;
       }
+    } else {
+        TRACE( 5, "awaited stray interrupt");
     }
     hide_interrupt = 0;
   }
@@ -658,3 +671,6 @@ fdc_isr( void)
   --isr_active;
 }
 
+
+
+
