@@ -15,19 +15,19 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-0.9.10/RCS/ftape-io.c,v $
+ $Source: /usr/src/distr/ftape-1.12/RCS/ftape-io.c,v $
  $Author: bas $
  *
- $Revision: 1.28 $
- $Date: 1994/02/20 15:26:55 $
- $State: Alpha $
+ $Revision: 1.30 $
+ $Date: 1994/05/26 23:43:45 $
+ $State: Beta $
  *
  *      This file contains some general functions
  *      for the QIC-40/80 floppy-tape driver for Linux.
  */
 
 static char RCSid[] =
-"$Id: ftape-io.c,v 1.28 1994/02/20 15:26:55 bas Alpha bas $";
+"$Id: ftape-io.c,v 1.30 1994/05/26 23:43:45 bas Beta bas $";
 
 
 #include <linux/errno.h>
@@ -212,7 +212,7 @@ ftape_parameter( int command)
 }
 
 /*      Wait for the drive to get ready.
- *      timeout time in seconds
+ *      timeout time in milli-seconds
  */
 int
 ftape_ready_wait( int timeout, int* status)
@@ -220,7 +220,6 @@ ftape_ready_wait( int timeout, int* status)
   TRACE_FUN( 8, "ftape_ready_wait");
   int result;
 
-  timeout *= 10;                /* 0.1 secs */
   for (;;) {
     result = ftape_report_raw_drive_status( status);
     if (result < 0) {
@@ -232,12 +231,13 @@ ftape_ready_wait( int timeout, int* status)
       TRACE_EXIT;
       return 0;
     }
-    if (--timeout < 0) {
+    if (timeout <= 0) {
       TRACE( 1, "timeout");
       TRACE_EXIT;
       return -ETIME;
     }
-    ftape_sleep( HZ / 10);      /* 0.1 second */
+    ftape_sleep( SECOND / 10);
+    timeout -= SECOND / 10;
   }
   TRACE_EXIT;
   return 0;
@@ -356,6 +356,10 @@ ftape_report_raw_drive_status( int *status)
       break;
     }
   } while (result < 0);
+  if (result >= 0 &&
+      *status & QIC_STATUS_READY && *status & QIC_STATUS_ERROR) {
+    TRACE( 2, "warning: error status set!");
+  }
   TRACE_EXIT;
   return result;
 }
@@ -381,7 +385,11 @@ ftape_report_drive_status( int *status)
   if (*status & QIC_STATUS_READY && *status & QIC_STATUS_ERROR) {
     /*  Let caller handle all other errors
      */
+#if 1
+    result = -EIO;
+#else
     result = 1 * 0;             /* WHAT'S THIS ???? FIX IT */
+#endif
   }
   TRACE_EXIT;
   return result;
@@ -544,7 +552,7 @@ ftape_seek_head_to_track( int track)
   int result;
 
   ftape_track = -1;             /* remains set in case of error */
-  if (track < 0 || track > tracks_per_tape) {
+  if (track < 0 || track >= tracks_per_tape) {
     TRACE( -1, "track out of bounds");
     result = -EINVAL;
   } else {
@@ -552,7 +560,7 @@ ftape_seek_head_to_track( int track)
     if (result < 0) {
       TRACE( 1, "ftape_command failed");
     } else {
-      result = ftape_parameter_wait( track + 2, 5, &status);
+      result = ftape_parameter_wait( track + 2, 5 * SECOND, &status);
       if (result < 0) {
         TRACE( 1, "ftape_parameter_wait failed");
       } else {
@@ -582,7 +590,7 @@ ftape_seek_to_eot( void)
   int result;
   int status;
 
-  result = ftape_ready_wait( 5, &status);
+  result = ftape_ready_wait( 5 * SECOND, &status);
   while ((status & QIC_STATUS_AT_EOT) == 0) {
     if (result < 0) {
       TRACE( 1, "failed");
@@ -593,7 +601,7 @@ ftape_seek_to_eot( void)
       TRACE_EXIT;
       return -EIO;
     }
-    result = ftape_command_wait( QIC_PHYSICAL_FORWARD, 85, &status);
+    result = ftape_command_wait( QIC_PHYSICAL_FORWARD, 85 * SECOND, &status);
   }
   TRACE_EXIT;
   return 0;
@@ -606,7 +614,7 @@ ftape_seek_to_bot( void)
   int result;
   int status;
 
-  result = ftape_ready_wait( 5, &status);
+  result = ftape_ready_wait( 5 * SECOND, &status);
   while ((status & QIC_STATUS_AT_BOT) == 0) {
     if (result < 0) {
       TRACE( 1, "failed");
@@ -617,7 +625,7 @@ ftape_seek_to_bot( void)
       TRACE_EXIT;
       return -EIO;
     }
-    result = ftape_command_wait( QIC_PHYSICAL_REVERSE, 85, &status);
+    result = ftape_command_wait( QIC_PHYSICAL_REVERSE, 85 * SECOND, &status);
   }
   TRACE_EXIT;
   return 0;
@@ -695,31 +703,40 @@ ftape_abort_operation( int* location)
   int i;
   int status;
 
-  for (i = 0; i < 2; ++i) {
-    if (runner_status == running) {
-      TRACE( 5, "aborting runner, waiting");
-      runner_status = do_abort;
-      result = fdc_interrupt_wait( 10 * SECOND);
-      if (result < 0) {
-        TRACE( 1, "fdc_interrupt_wait failed");
-        TRACE_EXIT;
-        return result;
-      }
+  if (runner_status == running) {
+    TRACE( 5, "aborting running runner, waiting");
+    runner_status = do_abort;
+    /* set timeout so that the tape will run into at logical EOT
+     * if we missed the last sector and there are no queue pulses.
+     */
+#if 0
+    result = fdc_interrupt_wait( 4 * SECOND);
+    if (result == 0) {
       result = ftape_smart_stop( &location_known, location);
-      if (result < 0) {
-        fdc_interrupt_wait( 2 * SECOND);
-        result = ftape_smart_stop( &location_known, location);
+      if (result == 0) {
+        runner_status = idle;
       }
-    } else {
+    } else if (result == -ETIME) {
       break;
+    } else {
+      TRACE( 1, "fdc_interrupt_wait failed");
+      TRACE_EXIT;
+      return result;
     }
+#else
+    result = ftape_smart_stop( &location_known, location);
+    if (result == 0) {
+      runner_status = idle;
+    }
+#endif
   }
   if (runner_status != idle) {
     if (runner_status == do_abort) {
       TRACE( 5, "forcing runner abort");
     }
     TRACE( 5, "stopping tape");
-    result = ftape_command_wait( QIC_STOP_TAPE, 5, &status);
+    result = ftape_command_wait( QIC_STOP_TAPE, 5 * SECOND, &status);
+    location_known = 0;
     runner_status = idle;
   }
   for (i = 0; i < NR_ITEMS( buffer); ++i) {
@@ -847,7 +864,7 @@ ftape_detach_drive( void)
 {
   TRACE_FUN( 8, "ftape_detach_drive");
 
-  TRACE( 3, "disabling tape drive and fdc");
+  TRACE( 5, "disabling tape drive and fdc");
   ftape_put_drive_to_sleep( drive_type);
   fdc_catch_stray_interrupts( 1);   /* one always comes */
   fdc_disable();
@@ -872,7 +889,9 @@ _ftape_open( void)
   do_floppy = fdc_isr;
   ftape_motor = 0;
   fdc_catch_stray_interrupts( 1); /* one always comes */
+  TRACE( 5, "resetting fdc");
   fdc_reset();                  /* init fdc */
+  TRACE( 5, "recalibrating");
   if (fdc_recalibrate() != 0) {
     TRACE( 1, "recalibrate failed");
     TRACE_EXIT;
@@ -882,7 +901,7 @@ _ftape_open( void)
    * Else try to find out what kind of drive is attached.
    */
   if (drive_type.wake_up != unknown_wake_up) {
-    TRACE( 3, "enabling tape drive and fdc");
+    TRACE( 5, "enabling tape drive and fdc");
     result = ftape_wakeup_drive( drive_type);
     if (result < 0) {
       TRACE( 1, "known wakeup method failed");
@@ -922,7 +941,7 @@ _ftape_open( void)
    *    First clear error status if present.
    */
   do {
-    ftape_ready_wait( 105, &status);
+    ftape_ready_wait( 105 * SECOND, &status);
     /*  Exit if no tape inserted
      */
     if ((status & QIC_STATUS_CARTRIDGE_PRESENT) == 0) {
@@ -994,7 +1013,7 @@ _ftape_open( void)
     if (!(status & QIC_STATUS_REFERENCED)) {
       TRACE( 5, "starting seek_load_point");
       ftape_seek_to_bot();      /* needed for Insight drive */
-      result = ftape_command_wait( QIC_SEEK_LOAD_POINT, 105, &status);
+      result = ftape_command_wait( QIC_SEEK_LOAD_POINT, 105 * SECOND, &status);
       if (result < 0) {
         TRACE( 1, "seek_load_point failed (command)");
         ftape_detach_drive();
