@@ -15,19 +15,19 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-0.9.10/RCS/ftape-read.c,v $
+ $Source: /usr/src/distr/ftape-1.12/RCS/ftape-read.c,v $
  $Author: bas $
  *
- $Revision: 1.5 $
- $Date: 1994/02/20 15:26:55 $
- $State: Alpha $
+ $Revision: 1.7 $
+ $Date: 1994/05/26 23:43:45 $
+ $State: Beta $
  *
  *      This file contains the reading code
  *      for the QIC-117 floppy-tape driver for Linux.
  */
 
 static char RCSid[] =
-"$Id: ftape-read.c,v 1.5 1994/02/20 15:26:55 bas Alpha bas $";
+"$Id: ftape-read.c,v 1.7 1994/05/26 23:43:45 bas Beta bas $";
 
 
 #include <linux/string.h>
@@ -197,14 +197,18 @@ read_segment( unsigned segment_id, byte* address, int* eof_mark,
     if (!read_done && buffer[ tail].status == reading) {
       if (buffer[ tail].segment_id == segment_id) {
         int result = wait_segment( reading);
-        if (result == -EINTR) {
-          TRACE_EXIT;
-          return result;
-        }
         if (result < 0) {
+          if (result == -EINTR) {
+            TRACE_EXIT;
+            return result;
+          }
           TRACE( 1, "wait_segment failed while reading");
+#if 0
           TRACE_EXIT;
           return -EIO;
+#else
+          ftape_abort_operation( &current_segment);
+#endif
         }
       } else {
         /*        We're reading the wrong segment, stop runner.
