@@ -15,19 +15,19 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-0.9.10/RCS/ftape-rw.c,v $
+ $Source: /usr/src/distr/ftape-1.12/RCS/ftape-rw.c,v $
  $Author: bas $
  *
- $Revision: 1.28 $
- $Date: 1994/02/20 15:26:55 $
- $State: Alpha $
+ $Revision: 1.30 $
+ $Date: 1994/05/26 23:43:45 $
+ $State: Beta $
  *
  *      This file contains some common code for the segment read and segment
  *      write routines for the QIC-117 floppy-tape driver for Linux.
  */
 
 static char RCSid[] =
-"$Id: ftape-rw.c,v 1.28 1994/02/20 15:26:55 bas Alpha bas $";
+"$Id: ftape-rw.c,v 1.30 1994/05/26 23:43:45 bas Beta bas $";
 
 
 #include <linux/string.h>
@@ -63,6 +63,7 @@ ftape_last_segment_struct ftape_last_segment;
 int header_segment_1 = -1;
 int header_segment_2 = -1;
 unsigned long bad_sector_map[ 4200]; /* max size for QIC-80, 307.5 ft tape */
+unsigned int fast_seek_segment_time = 400;
 
 /*      Local vars.
  */
@@ -207,7 +208,7 @@ setup_new_segment( buffer_struct* buff, unsigned int segment_id,
   static int old_ftape_state = idle;
   int skip = 0;
 
-  TRACEx3( 4, "%s segment %d (old = %d)",
+  TRACEx3( 5, "%s segment %d (old = %d)",
           (ftape_state == reading) ? "reading" : "writing",
           segment_id, old_segment_id);
   if (ftape_state != old_ftape_state) { /* when verifying */
@@ -217,12 +218,12 @@ setup_new_segment( buffer_struct* buff, unsigned int segment_id,
   if (segment_id == old_segment_id) {
     ++buff->retry;
     ++history.retries;
-    TRACEx1( 4, "setting up for retry nr %d", buff->retry);
+    TRACEx1( 5, "setting up for retry nr %d", buff->retry);
     if (offset < 0 && buff->skip > 0) { /* allow skip on retry */
       offset = buff->skip;
       count -= offset;
       skip = 1;
-      TRACEx1( 4, "skipping %d sectors", offset);
+      TRACEx1( 5, "skipping %d sectors", offset);
     }
   } else {
     buff->retry = 0;
@@ -272,39 +273,57 @@ ftape_read_id( int *location)
 {
   TRACE_FUN( 8, "ftape_read_id");
   int result;
+  int status;
   byte out[2];
-  int retry;
 
-  for (retry = 0; retry < 2; ++retry) {
-    out[ 0] = FDC_READID;
-    out[1] = FTAPE_UNIT;
-    result = fdc_command( out, 2);
-    if (result < 0) {
-      TRACE( 1, "fdc_command failed");
-      *location = -1;
-      break;
-    }
-    result = fdc_interrupt_wait( 10 * SECOND);
-    if (result < 0) {
-      TRACE( 1, "fdc_interrupt_wait failed");
-      *location = -1;
-      break;
-    }
-    /*    result set by isr, will be -1 on error.
+  /* Assume tape is running on entry, be able to handle
+   * situation where it stopped or is stopping.
+   */
+  *location = -1;               /* default is location not known */
+  out[ 0] = FDC_READID;
+  out[1] = FTAPE_UNIT;
+  result = fdc_command( out, 2);
+  if (result < 0) {
+    TRACE( 1, "fdc_command failed");
+  } else {
+    /* set timeout so that the tape will run to logical EOT if
+     * we missed the last sector and there are no INDEX cue pulses.
      */
-    if (fdc_sect == 0) {
-      *location = -1;
-      result = -EIO;            /* if too many retries */
-      TRACE( 5, "current location unknown");
-    } else {
-      *location = (segments_per_head * fdc_head
-                   + segments_per_cylinder * fdc_cyl
-                   + (fdc_sect - 1) / SECTORS_PER_SEGMENT);
+    result = fdc_interrupt_wait( 4 * SECOND);
+    if (result == -ETIME) {
+      ftape_report_drive_status( &status);
+      if (status & QIC_STATUS_READY) {
+        /* not running any more !
+         */
+        TRACE( 5, "tape has stopped");
+        if (((ftape_track & 1) == 0 && (status & QIC_STATUS_AT_EOT)) ||
+            ((ftape_track & 1) != 0 && (status & QIC_STATUS_AT_BOT))) {
+          /* Say we're passing last segment on track
+           */
+          *location = (ftape_track + 1) * segments_per_track - 1;
+        }
+      } else {
+        TRACE( 1, "weird, drive busy but no data");
+      }
       result = 0;
-      TRACEi( 5, "current location =", *location);
-      break;
+    } else if (result == 0) {
+      /*    fdc_sect is set by fdc_isr(), will be 0 on error.
+       */
+      if (fdc_sect != 0) {
+        *location = (segments_per_head * fdc_head
+                     + segments_per_cylinder * fdc_cyl
+                     + (fdc_sect - 1) / SECTORS_PER_SEGMENT);
+      }
+    } else {
+      /* Interrupted or otherwise failing fdc_interrupt_wait()
+       */
     }
   }
+  if (*location == -1) {
+    TRACE( 5, "no id found");
+  } else {
+    TRACEi( 5, "passing segment ", *location);
+  }
   TRACE_EXIT;
   return result;
 }
@@ -317,22 +336,38 @@ ftape_smart_stop( int* location_known, int* location)
   int result;
   int status;
 
-  /* Make sure fdc is ready. There's no information on how long
-   * this can take, but 100 usecs seems very long to me.
-   * If it still fails, the read_id that follows will fail and
-   * tell us so !
-   * If the tape is at it's end (drive not ready), the index cue
-   * pulses will make sure read_id returns with an error.
+  /* Abort current fdc operation if it's busy (probably read
+   * or write operation pending) with a reset.
    */
-  fdc_ready_wait( 100 /* usec */);
-  *location_known = (ftape_read_id( location) == 0);
-  if (*location < 0) {
-    *location_known = 0;
-  }
-  result = ftape_command_wait( QIC_STOP_TAPE, 5, &status);
+  result = fdc_ready_wait( 100 /* usec */);
   if (result < 0) {
-    TRACE( 1, "qic_stop_tape command_wait failed");
-    *location_known = 0;
+    TRACE( 1, "aborting fdc operation");
+    fdc_reset();
+  }
+  ftape_report_drive_status( &status);
+  if (status & QIC_STATUS_READY) {
+    /* Tape is not running any more.
+     */
+    TRACE( 5, "tape already halted");
+    if (((ftape_track & 1) == 0 && (status & QIC_STATUS_AT_EOT)) ||
+        ((ftape_track & 1) != 0 && (status & QIC_STATUS_AT_BOT))) {
+      /* Say we're passing last segment on track
+       */
+      *location = (ftape_track + 1) * segments_per_track - 1;
+      *location_known = 1;
+    } else {
+      *location_known = 0;
+    }
+  } else {
+    /* Tape is (was) still moving.
+     */
+    result = ftape_read_id( location);
+    *location_known = (*location != -1);
+    result = ftape_command_wait( QIC_STOP_TAPE, 5 * SECOND, &status);
+    if (result < 0) {
+      TRACE( 1, "problem stopping tape");
+      *location_known = 0;
+    }
   }
   if (*location_known) {
     TRACEi( 4, "tape stopped passing segment:", *location);
@@ -343,7 +378,6 @@ ftape_smart_stop( int* location_known, int* location)
   return result;
 }
 
-
 /*      Wait until runner has finished tail buffer.
  */
 int
@@ -375,19 +409,28 @@ wait_segment( buffer_state_enum state)
 }
 
 int
-seek_reverse( int count)
+fast_seek( int count)
 {
-  TRACE_FUN( 5, "seek_reverse");
+  TRACE_FUN( 5, "fast_seek");
   int result;
   int status;
 
-  result = ftape_ready_wait( 5, &status);  /* timeout for stop-tape command */
+  /* timeout for stop-tape command
+   */
+  result = ftape_ready_wait( 5 * SECOND, &status);
   if (result < 0) {
     TRACE( 1, "drive not ready");
     result = -EIO;
   } else {
     /* Issue this tape command first. */
-    result = ftape_command( QIC_SKIP_REVERSE);
+    if (count >= 0) {
+      TRACEx1( 5, "skipping %d segments", count);
+      result = ftape_command( QIC_SKIP_FORWARD);
+    } else {
+      TRACEx1( 5, "backing up %d segments", -count);
+      count = -count;
+      result = ftape_command( QIC_SKIP_REVERSE);
+    }
     if (result >= 0) {
       /* Issue the low nibble of the command. */
       result = ftape_parameter( 2 + (count & 0x0f));
@@ -395,7 +438,7 @@ seek_reverse( int count)
         /* Issue the high nibble and wait for the command to complete. */
         result = ftape_parameter( 2 + ((count >> 4) & 0x0f));
         if (result >= 0) {
-          result = ftape_ready_wait( 85, &status);
+          result = ftape_ready_wait( 85 * SECOND, &status);
         }
       }
     }
@@ -424,11 +467,7 @@ ftape_start_tape( int segment_id)
   static int last_segment = -1;
   int ids_read;
 
-#if 0
-  if (last_segment == segment_id) { /* might be a retry */
-#else
   if (buffer[ head].retry > 0) { /* this is a retry */
-#endif
     TRACEx1( 5, "seek segment %d (again)", segment_id);
   } else {
 #ifdef SLOW_STARTER
@@ -441,16 +480,17 @@ ftape_start_tape( int segment_id)
   last_segment = segment_id;
   do {
     if (!tape_running) {
-      result = ftape_ready_wait( 15, &status);
+      result = ftape_ready_wait( 15 * SECOND, &status);
       if (result < 0) {
         TRACEi( 1, "wait for ready failed with code", result);
         break;
       }
-    }
-    result = ftape_report_drive_status( &status);
-    if (result < 0) {
-      TRACEi( 1, "ftape_report_drive_status failed with code", result);
-      break;
+    } else {
+      result = ftape_report_drive_status( &status);
+      if (result < 0) {
+        TRACEi( 1, "ftape_report_drive_status failed with code", result);
+        break;
+      }
     }
     if (ftape_track != track) {
       /* current unknown or not equal to destination
@@ -511,7 +551,7 @@ ftape_start_tape( int segment_id)
     }
     if (rewind_gaps > 0) {        /* in or past target segment */
       if (tape_running) {
-        result = ftape_command_wait( QIC_STOP_TAPE, 5, &status);
+        result = ftape_command_wait( QIC_STOP_TAPE, 5 * SECOND, &status);
         if (result < 0) {
           TRACEi( 1, "stop tape failed failed with code", result);
           break;
@@ -520,11 +560,11 @@ ftape_start_tape( int segment_id)
         tape_running = 0;
       }
       TRACE( 5, "repositioning");
-      seek_reverse( rewind_gaps + reverse_margin +
-                   buffer[ head].start_offset - 1);
+      fast_seek( -(rewind_gaps + reverse_margin +
+                buffer[ head].start_offset - 1));
       ++history.rewinds;
       next_segment = segment_id - reverse_margin - buffer[ head].start_offset;
-      result = ftape_ready_wait( 15, &status);
+      result = ftape_ready_wait( 15 * SECOND, &status);
       if (result < 0) {
         TRACEi( 1, "wait for ready failed with code", result);
         break;
@@ -554,6 +594,29 @@ ftape_start_tape( int segment_id)
       if (result == 0 && current_segment >= 0) {
         next_segment = 1 + current_segment;
         TRACEi( 5, "skipping segment", current_segment);
+#ifdef FAST_SEEK
+        if (fast_seek_segment_time > 0 && segment_id - next_segment >= 8) {
+          result = ftape_command_wait( QIC_STOP_TAPE, 5 * SECOND, &status);
+          if (result >= 0) {
+            TRACE( 5, "tape stopped");
+            result = fast_seek( segment_id - next_segment - 3);
+            if (result == 0) {
+              result = ftape_ready_wait( 85 * SECOND, &status);
+            }
+            result = ftape_command_wait( QIC_STOP_TAPE, 5 * SECOND, &status);
+            if (result >= 0) {
+              TRACE( 5, "tape stopped");
+              ftape_command( QIC_LOGICAL_FORWARD);
+              TRACEi( 5, "starting tape before segment", next_segment);
+              tape_running = 1;
+              result = ftape_read_id( &current_segment);
+              if (result == 0 && current_segment >= 0) {
+                next_segment = 1 + current_segment;
+              }
+            }
+          }
+        }
+#endif
       } else {
         if (ids_read++ > 25) {  /* must be enough */
           TRACE( 1, "read_id failed completely");
