@@ -15,19 +15,19 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /usr/src/distr/ftape-0.9.10/RCS/ftape-write.c,v $
+ $Source: /usr/src/distr/ftape-1.12/RCS/ftape-write.c,v $
  $Author: bas $
  *
- $Revision: 1.5 $
- $Date: 1994/02/20 15:26:55 $
- $State: Alpha $
+ $Revision: 1.7 $
+ $Date: 1994/05/26 23:43:45 $
+ $State: Beta $
  *
  *      This file contains the writing code
  *      for the QIC-117 floppy-tape driver for Linux.
  */
 
 static char RCSid[] =
-"$Id: ftape-write.c,v 1.5 1994/02/20 15:26:55 bas Alpha bas $";
+"$Id: ftape-write.c,v 1.7 1994/05/26 23:43:45 bas Beta bas $";
 
 
 #include <linux/string.h>
@@ -227,7 +227,7 @@ write_segment( unsigned segment_id, byte* address, int flushing)
     /*  setup for a retry
      */
     buffer[ tail].status = full;
-    result = -EAGAIN;           /* force retry */
+    bytes_written = -EAGAIN;    /* force retry */
   } else if (buffer[ tail].status != empty) {
     TRACE( 1, "wait for empty segment failed");
     result = -EIO;
@@ -261,10 +261,11 @@ write_segment( unsigned segment_id, byte* address, int flushing)
       runner_status = idle;       /* aborted ? */
     }
   }
-  /*  Don't start tape if not running already and empty segment.
+  /*  Don't start tape if runner idle and segment empty.
    */
-  if (runner_status != idle || bad_sector_map[ segment_id] != EMPTY_SEGMENT) {
-    if (result >= 0 && buffer[ tail].status == empty) {
+  if (result >= 0 && !(runner_status == idle &&
+                       bad_sector_map[ segment_id] == EMPTY_SEGMENT)) {
+    if (buffer[ tail].status == empty) {
       /*    now at least one buffer is empty, fill it with our data.
        *    skip bad sectors and generate ecc.
        *    copy_and_gen_ecc return nr of bytes written,
@@ -303,7 +304,7 @@ _write_segment( unsigned int segment_id, byte* buffer, int flush)
 {
   TRACE_FUN( 5, "_write_segment");
   int retry = 0;
-  int result, temp_result;
+  int result;
 
   history.used |= 2;
   for (;;) {
@@ -320,9 +321,6 @@ _write_segment( unsigned int segment_id, byte* buffer, int flush)
           result = -EIO;        /* give up */
           break;
         }
-        temp_result = ftape_smart_stop( &location_known, &current_segment);
-        runner_status = idle;
-        temp_result = start_writing (WRITE_MULTI);
       } else {
         TRACEi( 1, "write_segment failed, error:", -result);
         break;
